`timescale 1ns / 1ps
//*****************************************************************************
// File Name            : mb_mcs_sys.v
//-----------------------------------------------------------------------------
// Function             : MicroBlaze MAC system 
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo 
//-----------------------------------------------------------------------------
// History
// -.-- 2013/04/19
// -.-- 2013/07/08
//*****************************************************************************

module mb_mcs_sys(  clk_100m,Reset, UART_Rx, UART_Tx,led,scl,sda,dip_sw,lcd_co,mon
    );

  input  clk_100m;      //input clock 100MHz
  input  Reset;    //reset Hi=reset
  input  UART_Rx;  //UART rx 115200bps
  output UART_Tx; //UART tx 115200bps
  output [3 : 0] led;   //led Hi=on
  input  [3 : 0] dip_sw; //input dip_sw
  inout          scl; //i2c clk
  inout          sda; //i2c data
  output         lcd_co;   
  output         mon;
   
  // 
  parameter p_reg_addr_low =  32'hC0000000;
  parameter p_reg_addr_hi  =  32'hC0000007;
  parameter p_bram_addr_low = 32'hC0001000;
  parameter p_bram_addr_hi  = 32'hC00017FF;
  // 
  parameter p_wb_addr_low  =  32'hD0000000;
  parameter p_wb_addr_hi   =  32'hEFFFFFFF;
  parameter p_wb_pwm0_low   =  p_wb_addr_low + 0;
  parameter p_wb_pwm0_hi    =  p_wb_addr_low + 15;
  parameter p_wb_pwm1_low   =  p_wb_addr_low + 16;
  parameter p_wb_pwm1_hi    =  p_wb_addr_low + 31;
  parameter p_wb_i2c_low   =  p_wb_addr_low + 64;
  parameter p_wb_i2c_hi   =  p_wb_addr_low + 127;
  //IO Bus
  wire IO_Addr_Strobe;
  wire IO_Read_Strobe;
  wire IO_Write_Strobe;
  wire [31 : 0] IO_Read_Data;
  wire [31 : 0] IO_Read_Data_reg;
  wire [31 : 0] IO_Read_Data_bram;
  wire [31 : 0] IO_Read_Data_wb;
  wire [31 : 0] IO_Address;
  wire [3 : 0]  IO_Byte_Enable;
  wire [31 : 0] IO_Write_Data;
  wire IO_Ready;
  wire IO_Ready_reg;
  wire IO_Ready_bram;
  wire IO_Ready_wb;
  wire iobus_reg_sel;
  wire iobus_bram_sel;  
  wire iobus_wb_sel;   
  //timmer
  wire FIT1_Interrupt;
  wire FIT1_Toggle;    
  wire PIT1_Interrupt; 
  wire PIT1_Toggle; 
  //GPO   
  wire [31 : 0] GPO1;
  wire [31 : 0] GPI1;
  //GPI
  wire GPI1_Interrupt;
  //Interrupt controller
  wire INTC_Interrupt;
  wire INTC_IRQ;
  //iobus_reg
  wire [31:0] data_in;
  wire [3:0]  data_in_en;
  wire [31:0] data_out;
  wire [3:0]  data_out_en;
  //user_module
  wire [7:0] chk_data; 
  //block ram port-B
  wire  [3:0]web;
  wire  [8 : 0] addrb;
  wire  [31 : 0] dinb;
  wire  [31 : 0] doutb;
  //Wishbone bus microblaze -> IP
  wire         WB_CLK;         // clock 
  wire         WB_RST;         // reset (asynchronous active hi)
  wire         WB_CYC;         // cycle
  wire         WB_STB;         // strobe
  wire  [31:0] WB_ADR;         // address adr_i
  wire         WB_WE;          // write enable
  wire  [31:0] WB_DAT_I;          // data output
  wire  [31:0] WB_DAT_O;          // data input
  wire  [3:0]  WB_SEL;         // byte enable
  wire         WB_ACK;         // normal bus termination
  wire         WB_ERR;         // err bus termination
  wire         WB_RTY;         // rtry
  //pwm
  wire         cyc_i_pwm0;         // cycle
  wire         stb_i_pwm0;         // strobe
  wire         we_i_pwm0;          // write enable
  wire  [15:0] adr_i_pwm0;         // address adr_i
  wire  [15:0] dat_o_pwm0;   // data input
  wire         ack_o_pwm0;         // normal bus termination
  wire         wb_pwm0_sel;
  wire         cyc_i_pwm1;         // cycle
  wire         stb_i_pwm1;         // strobe
  wire  [15:0] adr_i_pwm1;         // address adr_i
  wire         we_i_pwm1;          // write enable
  wire  [15:0] dat_o_pwm1;   // data input
  wire         ack_o_pwm1;         // normal bus termination
  wire         wb_pwm1_sel;
  //i2c
  wire         cyc_i_i2c;         // cycle
  wire         stb_i_i2c;         // strobe
  wire         we_i_i2c;          // write enable
  wire  [2:0] adr_i_i2c;         // address adr_i
  wire  [7:0] dat_o_i2c;   // data input
  wire         ack_o_i2c;         // normal bus termination
  wire         wb_i2c_sel;
  wire         inta_i2c;
               
  //opencores PWM signals
  wire  i_extclk;
  wire  [15:0]i_DC;
  wire  i_valid_DC;  
  wire  o_pwm0;   
   
  //opencores I2C signals
  wire rstn;
  wire scl_pad_i;
  wire scl_pad_o;
  wire scl_padoe_o;
  wire sda_pad_i;
  wire sda_pad_o;
  wire sda_padoe_o;

  //mon
  reg mon;
  //clk
  wire Clk;
  reg [7:0]clk_cnt = 8'h00;
  reg [7:0] reset_cnt = 8'h00 ;
  reg dco_reset = 1'b1;
  reg scl_mon ;
  reg scl_o_mon ;
  reg scl_oe_mon ;
  reg sda_mon ;
  reg sda_oe_mon ;
  reg sda_o_mon ;
  reg sda_i_mon ;
  reg scl_fd1; 
  reg scl_o_fd1 ;
  reg scl_oe_fd1 ;
  reg sda_fd1 ;
  reg sda_oe_fd1 ;
  reg sda_o_fd1 ;
  reg sda_i_fd1 ;
  reg scl_fd2; 
  reg scl_o_fd2 ;
  reg scl_oe_fd2 ;
  reg sda_fd2 ;
  reg sda_oe_fd2 ;
  reg sda_o_fd2 ;
  reg sda_i_fd2 ;


always @ (posedge clk_100m )
  if (clk_cnt==8'd99)
    clk_cnt <= 8'h0;
  else
    clk_cnt <= clk_cnt + 8'd1;

   
   assign Clk = clk_100m;
   


always @ (posedge Clk or posedge  Reset )
  if ( Reset==1'b1)
    reset_cnt <= 8'h00;
  else
    if (reset_cnt==8'hff)
      reset_cnt <= 8'hff;
    else
      reset_cnt <= reset_cnt + 8'h01;

always @ (posedge Clk or posedge  Reset )
  if ( Reset==1'b1)
    dco_reset <= 1'b1;
  else
    if (reset_cnt==8'hff)
      dco_reset <= 1'b0;
    else
      dco_reset <= 1'b1;
   
  //LED
  
  //assign led = {data_out[1:0],GPO1[1:0]};
  assign led = {2'b00,o_pwm1,o_pwm0};
  //
  assign lcd_co = o_pwm1;
   
  //Mڑ
  assign rstn = ~Reset;
  assign GPI1 = {28'h0000000,dip_sw};
  assign data_in = data_out;
  assign data_in_en = data_out_en; 
  assign chk_data = data_out[7:0];  
  assign INTC_Interrupt = 1'b0;
   

  //ӉHIM
  assign iobus_reg_sel  = ((IO_Address>=p_reg_addr_low)&&(IO_Address<=p_reg_addr_hi))? 1'b1:1'b0;
  assign iobus_bram_sel = ((IO_Address>=p_bram_addr_low)&&(IO_Address<=p_bram_addr_hi))? 1'b1:1'b0;   
  assign iobus_wb_sel  =  ((IO_Address>=p_wb_addr_low)&&(IO_Address<=p_wb_addr_hi))? 1'b1:1'b0;
  //Ǐof[^̑I       
  assign IO_Read_Data = (iobus_reg_sel==1'b1)?  IO_Read_Data_reg:
                        (iobus_bram_sel==1'b1)? IO_Read_Data_bram:
                        (iobus_wb_sel==1'b1)?   IO_Read_Data_wb:
                                                32'h00000000;
  //ReadyM̑I  
  assign IO_Ready =    (iobus_reg_sel==1'b1)?  IO_Ready_reg:
                       (iobus_bram_sel==1'b1)? IO_Ready_bram:
                       (iobus_wb_sel==1'b1)?  IO_Ready_wb:
                                               1'b0;

//Microblaze MCS̃CX^X
mb_mcs mcs_0 (
  .Clk(Clk), // input Clk
  .Reset(dco_reset), // input Reset
  .IO_Addr_Strobe(IO_Addr_Strobe), // output IO_Addr_Strobe
  .IO_Read_Strobe(IO_Read_Strobe), // output IO_Read_Strobe
  .IO_Write_Strobe(IO_Write_Strobe), // output IO_Write_Strobe
  .IO_Address(IO_Address), // output [31 : 0] IO_Address
  .IO_Byte_Enable(IO_Byte_Enable), // output [3 : 0] IO_Byte_Enable
  .IO_Write_Data(IO_Write_Data), // output [31 : 0] IO_Write_Data
  .IO_Read_Data(IO_Read_Data), // input [31 : 0] IO_Read_Data
  .IO_Ready(IO_Ready), // input IO_Ready
  .UART_Rx(UART_Rx), // input UART_Rx
  .UART_Tx(UART_Tx), // output UART_Tx
  .FIT1_Interrupt(FIT1_Interrupt), // output FIT1_Interrupt
  .FIT1_Toggle(FIT1_Toggle), // output FIT1_Toggle
  .PIT1_Interrupt(PIT1_Interrupt), // output PIT1_Interrupt
  .PIT1_Toggle(PIT1_Toggle), // output PIT1_Toggle
  .GPO1(GPO1), // output [31 : 0] GPO1
  .GPI1(GPI1), // input [31 : 0] GPI1
  .GPI1_Interrupt(GPI1_Interrupt), // output GPI1_Interrupt
  .INTC_Interrupt(INTC_Interrupt), // input [0 : 0] INTC_Interrupt
  .INTC_IRQ(INTC_IRQ) // output INTC_IRQ
); 
 
//ݒ背WX^̃CX^X
iobus_reg
#(p_reg_addr_low, p_reg_addr_hi)
iobus_reg_0 (
  .Clk(Clk), // input Clk
  .Reset(dco_reset), // input Reset
  .IO_Addr_Strobe(IO_Addr_Strobe), // output IO_Addr_Strobe
  .IO_Read_Strobe(IO_Read_Strobe), // output IO_Read_Strobe
  .IO_Write_Strobe(IO_Write_Strobe), // output IO_Write_Strobe
  .IO_Address(IO_Address), // output [31 : 0] IO_Address
  .IO_Byte_Enable(IO_Byte_Enable), // output [3 : 0] IO_Byte_Enable
  .IO_Write_Data(IO_Write_Data), // output [31 : 0] IO_Write_Data
  .IO_Read_Data(IO_Read_Data_reg), // input [31 : 0] IO_Read_Data
  .IO_Ready(IO_Ready_reg), // input IO_Ready
  //input 
  .data_in(data_in), // input data
  .data_in_en(data_in_en), // input data enable
  //output 
  .data_out(data_out), // output data
  .data_out_en(data_out_en) // output data enable
);

//ubNRAMC^[tF[X̃CX^X
iobus_bram
 #(p_bram_addr_low, p_bram_addr_hi)
iobus_bram_0 (
  .Clk(Clk), // input Clk
  .Reset(dco_reset), // input Reset
  .IO_Addr_Strobe(IO_Addr_Strobe), // output IO_Addr_Strobe
  .IO_Read_Strobe(IO_Read_Strobe), // output IO_Read_Strobe
  .IO_Write_Strobe(IO_Write_Strobe), // output IO_Write_Strobe
  .IO_Address(IO_Address), // output [31 : 0] IO_Address
  .IO_Byte_Enable(IO_Byte_Enable), // output [3 : 0] IO_Byte_Enable
  .IO_Write_Data(IO_Write_Data), // output [31 : 0] IO_Write_Data
  .IO_Read_Data(IO_Read_Data_bram), // input [31 : 0] IO_Read_Data
  .IO_Ready(IO_Ready_bram), // input IO_Ready
  //BRAM Port-B                  
  .clkb(Clk), // input clkb
  .web(web), // input [3 : 0] web
  .addrb(addrb), // input [8 : 0] addrb
  .dinb(dinb), // input [31 : 0] dinb
  .doutb(doutb) // output [31 : 0] doutb
);
   
user_module user_module_0 (
  .Clk(Clk), // input Clk
  .Reset(dco_reset), // input Reset
  .chk_data(chk_data),
  //BRAM Port-B                  
  .clk(clkb), // input clkb
  .we(web), // input [3 : 0] web
  .addr(addrb), // input [8 : 0] addrb
  .dout(dinb) // output [31 : 0] doutb
);

iobus2wb 
#(p_wb_addr_low, p_wb_addr_hi)
iobus2wb(
  .Clk(Clk), // input Clk
  .Reset(dco_reset), // input Reset
  .IO_Addr_Strobe(IO_Addr_Strobe), // output IO_Addr_Strobe
  .IO_Read_Strobe(IO_Read_Strobe), // output IO_Read_Strobe
  .IO_Write_Strobe(IO_Write_Strobe), // output IO_Write_Strobe
  .IO_Address(IO_Address), // output [31 : 0] IO_Address
  .IO_Byte_Enable(IO_Byte_Enable), // output [3 : 0] IO_Byte_Enable
  .IO_Write_Data(IO_Write_Data), // output [31 : 0] IO_Write_Data
  .IO_Read_Data(IO_Read_Data_wb), // input [31 : 0] IO_Read_Data
  .IO_Ready(IO_Ready_wb), // input IO_Ready
  //wishbone MicorBlaze -> IP
  .CLK_O(WB_CLK),         // clock
  .RST_O(WB_RST),         // reset 
  .CYC_O(WB_CYC),         // cycle
  .STB_O(WB_STB),         // strobe
  .ADR_O(WB_ADR),         // address adr_i
  .SEL_O(WB_SEL),         // byte_sel
  .WE_O( WB_WE),          // write enable
  .DAT_O(WB_DAT_I),       // data output
  .DAT_I(WB_DAT_O),       // data input
  .ACK_I(WB_ACK),         // normal bus termination
  .ERR_I(WB_ERR),         // error bus termination
  .RTY_I(WB_RTY)          // retry
                  );
     
//opencores simple_gpio    
 PWM PWM_0(
  .i_wb_clk(WB_CLK),         // clock
  .i_wb_rst(WB_RST),        // reset (asynchronous active hi)
  .i_wb_cyc(cyc_i_pwm0),    // cycle
  .i_wb_stb(stb_i_pwm0),    // strobe
  .i_wb_adr(adr_i_pwm0),         // address adr_i
  .i_wb_we(we_i_pwm0),      // write enable
  .i_wb_data(WB_DAT_I[15:0]),     // data output
  .o_wb_data(dat_o_pwm0),    // data input
  .o_wb_ack(ack_o_pwm0),    // normal bus termination
  .i_extclk(i_extclk),
  .i_DC(i_DC),
  .i_valid_DC(i_valid_DC),
  .o_pwm(o_pwm0)
                         ); 
 PWM PWM_1(
  .i_wb_clk(WB_CLK),       // clock
  .i_wb_rst(WB_RST),       // reset (asynchronous active hi)
  .i_wb_cyc(cyc_i_pwm1),    // cycle
  .i_wb_stb(stb_i_pwm1),    // strobe
  .i_wb_adr(adr_i_pwm1), // address adr_i
  .i_wb_we(we_i_pwm1),      // write enable
  .i_wb_data(WB_DAT_I[15:0]),     // data output
  .o_wb_data(dat_o_pwm1),   // data input
  .o_wb_ack(ack_o_pwm1),    // normal bus termination
  .i_extclk(i_extclk),
  .i_DC(i_DC),
  .i_valid_DC(i_valid_DC),
  .o_pwm(o_pwm1)
                         );


 i2c_master_top i2c_top (
  // wishbone interface
  .wb_clk_i(WB_CLK),
  .wb_rst_i(WB_RST),
  .arst_i(rstn),
  .wb_adr_i(adr_i_i2c),
  .wb_dat_i(WB_DAT_I[7:0]),
  .wb_dat_o(dat_o_i2c),
  .wb_we_i(we_i_i2c),
  .wb_stb_i(stb_i_i2c),
  .wb_cyc_i(cyc_i_i2c),
  .wb_ack_o(ack_o_i2c),
  .wb_inta_o(inta_i2c),

// i2c signals
  .scl_pad_i(scl_pad_i),
  .scl_pad_o(scl_pad_o),
  .scl_padoen_o(scl_padoe_o),
  .sda_pad_i(sda_pad_i),
  .sda_pad_o(sda_pad_o),
  .sda_padoen_o(sda_padoe_o)
           );
   
//WihsBoneBus interconnect
   assign wb_pwm0_sel = ((WB_ADR>=p_wb_pwm0_low)&&(WB_ADR<=p_wb_pwm0_hi))? 1'b1:1'b0;
   assign cyc_i_pwm0 =(wb_pwm0_sel==1'b1)?WB_CYC:1'b0;
   assign stb_i_pwm0 =(wb_pwm0_sel==1'b1)?WB_STB:1'b0;
   assign adr_i_pwm0 ={12'h000,WB_ADR[3:0]};
   assign we_i_pwm0  =(wb_pwm0_sel==1'b1)?WB_WE:1'b0;
   
   assign wb_pwm1_sel = ((WB_ADR>=p_wb_pwm1_low)&&(WB_ADR<=p_wb_pwm1_hi))? 1'b1:1'b0;
   assign cyc_i_pwm1 =(wb_pwm1_sel==1'b1)?WB_CYC:1'b0;
   assign stb_i_pwm1 =(wb_pwm1_sel==1'b1)?WB_STB:1'b0;
   assign adr_i_pwm1 ={12'h000,WB_ADR[3:0]};
   assign we_i_pwm1  =(wb_pwm1_sel==1'b1)?WB_WE:1'b0;

                         
   assign wb_i2c_sel = ((WB_ADR>=p_wb_i2c_low)&&(WB_ADR<=p_wb_i2c_hi))? 1'b1:1'b0;
   assign cyc_i_i2c =(wb_i2c_sel==1'b1)?WB_CYC:1'b0;
   assign stb_i_i2c =(wb_i2c_sel==1'b1)?WB_STB:1'b0;
   assign adr_i_i2c =WB_ADR[2:0];
   assign we_i_i2c  =(wb_i2c_sel==1'b1)?WB_WE:1'b0;
                         
   assign WB_DAT_O =(wb_pwm0_sel==1'b1)?{dat_o_pwm0,dat_o_pwm0}:
                    (wb_pwm1_sel==1'b1)?{dat_o_pwm1,dat_o_pwm1}:
                    (wb_i2c_sel==1'b1)?{dat_o_i2c,dat_o_i2c,dat_o_i2c,dat_o_i2c}:
                                        32'h00000000;
   assign WB_ACK = (wb_pwm0_sel==1'b1)?ack_o_pwm0:
                   (wb_pwm1_sel==1'b1)?ack_o_pwm1:
                   (wb_i2c_sel==1'b1)?ack_o_i2c:
                                        WB_STB;
   
   assign WB_ERR = 1'b0;
   assign WB_RTY = 1'b0;
   
// PWM inputs
   assign i_extclk =1'b0;
   assign i_DC = 16'h0000;
   assign i_valid_DC =1'b0;    
   
// I2C
    assign scl = scl_padoe_o ? 1'bz : scl_pad_o;
    assign sda = sda_padoe_o ? 1'bz: sda_pad_o;
    assign scl_pad_i = scl;
    assign sda_pad_i = sda;

//mon
   
always @ (posedge clk_100m )
  mon <= sda_pad_i^scl_mon^scl_o_mon^scl_oe_mon^sda_mon^sda_oe_mon^sda_o_mon;
   

always @ (posedge clk_100m  )
    begin
      scl_fd1    <= scl;
      scl_oe_fd1 <= scl_padoe_o;
      scl_o_fd1  <= scl_pad_o;
      sda_fd1    <= sda; 
      sda_oe_fd1 <= sda_padoe_o;
      sda_o_fd1  <= sda_pad_o;
    end
   
always @ (posedge clk_100m  )
    begin
      scl_fd2    <= scl_fd1;
      scl_oe_fd2 <= scl_oe_fd1;
      scl_o_fd2  <= scl_o_fd1;
      sda_fd2    <= sda_fd1; 
      sda_oe_fd2 <= sda_oe_fd1;
      sda_o_fd2  <= sda_o_fd1;
    end
   
always @ (posedge clk_100m  )
    begin
      scl_mon    <= scl_fd2;
      scl_o_mon  <= scl_o_fd2;
      scl_oe_mon <= scl_oe_fd2;
      sda_mon    <= sda_fd2;
      sda_oe_mon <= sda_oe_fd2;
      sda_o_mon  <= sda_o_fd2;
    end
//                          
endmodule
